import uwsgi
import socket
import errno

def send_request(env, client):

	client.setblocking(1)

	client.send(b"GET /intl/it_it/images/logo.gif HTTP/1.0\r\n")
	client.send(b"Host: www.google.it\r\n\r\n")

	uwsgi.green_schedule()

	while 1:
		uwsgi.green_wait_fdread(client.fileno(), 10)
		buf = client.recv(4096)
		if len(buf) == 0:
			break
		else:
			yield buf


def application(env, start_response):

	s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	s.setblocking(0)

	c = s.connect_ex(('74.125.77.104', 80))
	if c == errno.EINPROGRESS:
		uwsgi.green_wait_fdwrite(s.fileno(), 10)
		for r in send_request(env, s):
			yield r
	elif c == errno.EISCONN: 
		for r in send_request(env, s):
			yield r
	else:
		uwsgi.green_schedule()
		start_response( '500 Internal Server Error', [ ('Content-Type', 'text/html')])
		yield "Internal Server Error"

	s.close()



import uwsgi
import time

def application(env, start_response):

	start_response('200 Ok', [ ('Content-Type','text/html') ] )

	yield "I am the worker %d<br/>" % uwsgi.worker_id()

	grunt = uwsgi.grunt()
	
	if grunt is None:
		print "worker %d detached" % uwsgi.worker_id()
	else:
		yield "And now i am the grunt with a fix worker id of %d<br/>" % uwsgi.worker_id()
		time.sleep(2)
		yield "Now, i will start a very slow task...<br/>"
		for i in xrange(1,10):
			yield "waiting for %d seconds<br/>" % i
			time.sleep(i)
	
		



import uwsgi

def send_request(env, client):

	uwsgi.send(client, b"GET /intl/it_it/images/logo.gif HTTP/1.0\r\n")

	# test for suspend/resume
	uwsgi.suspend()

	uwsgi.send(client, b"Host: www.google.it\r\n\r\n")


	while 1:
		yield uwsgi.wait_fd_read(client, 2)
		if env['x-wsgiorg.fdevent.timeout']:
			return

		buf = uwsgi.recv(client, 4096)
		if buf:
			yield buf
		else:
			break


def application(env, start_response):

	c = uwsgi.async_connect('74.125.232.115:80')

	# wait for connection
	yield uwsgi.wait_fd_write(c, 2)
	
	if env['x-wsgiorg.fdevent.timeout']:
		uwsgi.close(c)
		raise StopIteration

	if uwsgi.is_connected(c):
		for r in send_request(env, c):
			yield r
	else:
		start_response( '500 Internal Server Error', [ ('Content-Type', 'text/html')])
		yield "Internal Server Error"

	uwsgi.close(c)



class AppClass(object):

    def __call__(self, environ):
        status = b'200 OK'
        headers = [(b'Content-type', b'text/plain')]
        body = [b'Hello world!\n']
        return body, status, headers

application = AppClass()
